// WinAlert.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

public class WinAlert {

/// WINALERT VARIABLES
    public static final Bitmap errorIcon = Bitmap.bitmapNamed("erroricon.gif");
    public static final Bitmap alertIcon = Bitmap.bitmapNamed("alerticon.gif");
    String title;

/// ORIGINAL ALERT VARIABLES - Stolen from the Alert.java class
    /** Value returned when the user chooses the default option. */
    public final static int DEFAULT_OPTION = 1;

    /** Value returned when the user chooses the 2nd option. */
    public final static int SECOND_OPTION = 2;

    /** Value returned when the user chooses the 3rd option. */
    public final static int THIRD_OPTION = 3;


    private final int MIN_WIDTH          =  200;
    private final int WIDTH_MARGIN       =  12;
    private final int BOTTOM_MARGIN      =  6;
    private final int BUTTON_HORIZ_MARGIN = 6;

    static  final String DEFAULT_ACTION="performDefaultAction";
    static  final String SECOND_ACTION="performSecondAction";
    static  final String THIRD_ACTION="performThirdAction";
    private final int MIN_BUTTON_WIDTH=50;

    ExternalWindow window;
    AlertContentView  contentView;
    TextView  messageTextView;
    Button defaultButton,secondButton,thirdButton;
    int result;
    Button bitmapContainer;

    public static int runAlertExternally(Image image, String title,
                        String message, String defaultOption,
                        String secondOption, String thirdOption) {
        WinAlert alert = new WinAlert( title,
                                 message,
                                 defaultOption,
                                 secondOption,
                                 thirdOption);
        alert.setImage(image);
        return alert.run();
    }

    /** Creates an alert. Negative option and alternateOption are optional and
      * can be null.
      */
    private WinAlert(String title, String message, String positiveOption,
                 String negativeOption,String alternateOption) {
        super();
        Size sz;
        Hashtable defaultAttributes;
        TextParagraphFormat format;

        this.title = title;

        contentView = new AlertContentView(this,0,0,100,100);
        contentView.setVertResizeInstruction(View.HEIGHT_CAN_CHANGE);
        contentView.setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);

        bitmapContainer = new WinButton(12, 12, 100, 100);
        bitmapContainer.setEnabled(false);
        bitmapContainer.setBordered(false);
        ((WinButton)bitmapContainer).setSelectable(false);

        sz = bitmapContainer.minSize();
        bitmapContainer.sizeTo(sz.width,sz.height);
        contentView.addSubview(bitmapContainer);

        messageTextView  = new TextView(0,0,100,20);
        messageTextView.setEditable(false);
        messageTextView.setSelectable(false);
        messageTextView.setString(message);
        messageTextView.setTransparent(true);
        defaultAttributes = messageTextView.defaultAttributes();
        format = (TextParagraphFormat) defaultAttributes.get(TextView.PARAGRAPH_FORMAT_KEY);
        defaultAttributes = (Hashtable) defaultAttributes.clone();
        format = (TextParagraphFormat) format.clone();
        format.setLeftMargin(0);
        format.setLeftIndent(0);
        format.setRightMargin(0);
        defaultAttributes.put(TextView.PARAGRAPH_FORMAT_KEY, format);
        messageTextView.setDefaultAttributes(defaultAttributes);
        contentView.addSubview(messageTextView);

        defaultButton = new WinButton(0,0,100,20);
        defaultButton.setTitle(positiveOption);
        defaultButton.setTarget(contentView);
        defaultButton.setCommand(DEFAULT_ACTION);
        ((WinButton)defaultButton).setDefault(true);
        contentView.addSubview(defaultButton);

        if( negativeOption != null ) {
            secondButton = new WinButton(0,0,100,20);
            secondButton.setTitle(negativeOption);
            secondButton.setTarget(contentView);
            secondButton.setCommand(SECOND_ACTION);
            contentView.addSubview(secondButton);
            secondButton.setCommandForKey("click", null, KeyEvent.ESCAPE_KEY,
                                    KeyEvent.NO_MODIFIERS_MASK,
                                    View.WHEN_IN_MAIN_WINDOW);  
        }

        if( alternateOption != null ) {
            thirdButton = new WinButton(0,0,100,20);
            thirdButton.setTitle(alternateOption);
            thirdButton.setTarget(contentView);
            thirdButton.setCommand(THIRD_ACTION);
            contentView.addSubview(thirdButton);
        }

    }

    private void setImage(Image anImage) {
        Size sz;

        bitmapContainer.setImage(anImage);
        sz = bitmapContainer.minSize();
        bitmapContainer.sizeTo(sz.width,sz.height);
    }

    private void calculateLayout() {
        FontMetrics fm;
        int buttonRowWidth;
        int minWidth = MIN_WIDTH;
        int centeredButtonSpacing = 0;
        int buttonOffset = 0;

        /* Calculate the required width */

        /* Message */
        Size reqSize  = requiredSizeToPreserveLinesInTextView(messageTextView);
        messageTextView.sizeTo(reqSize.width,reqSize.height);

        minWidth =  max(minWidth, reqSize.width + (2 * WIDTH_MARGIN));

        if(bitmapContainer.image() != null) {
            minWidth += bitmapContainer.bounds.width + WIDTH_MARGIN;
        }

        /* Buttons */
        Size buttonSize = new Size();
        int buttonCount = 1;
        fm = new FontMetrics(defaultButton.font());
        buttonSize.width  = max(fm.stringWidth(defaultButton.title()) + 10,
                                MIN_BUTTON_WIDTH);
        buttonSize.height = 24; 
        buttonRowWidth = buttonSize.width;

        if( secondButton != null ) {
            buttonSize.width = max(buttonSize.width, fm.stringWidth(secondButton.title()) + 10);
            buttonRowWidth += buttonSize.width + BUTTON_HORIZ_MARGIN;
            buttonCount++;
        }

        if( thirdButton != null ) {
            buttonSize.width = max(buttonSize.width, fm.stringWidth(thirdButton.title()) + 10);
            buttonRowWidth += buttonSize.width + BUTTON_HORIZ_MARGIN;
            buttonCount++;
        }

        minWidth = max(minWidth,buttonRowWidth);
        buttonOffset = (minWidth - buttonRowWidth) / 2;

        /* Let's make the layout */
        Rect rect = new Rect();

        /* Image  */
        rect.x = WIDTH_MARGIN;
        rect.y = WIDTH_MARGIN;
        if( bitmapContainer.image() != null ) {
            if( bitmapContainer.superview() == null )
                contentView.addSubview(bitmapContainer);
            bitmapContainer.moveTo(rect.x,rect.y);
        } else
            bitmapContainer.removeFromSuperview();

        /* Message */
        rect.y = WIDTH_MARGIN;
        // If this is a single line message, then center it with the 
        // icon. Otherwise, we'll be flush top.
        if(messageTextView.bounds.height <= fm.stringHeight())
            rect.y += 11;
        rect.x = bitmapContainer.bounds.maxY() + WIDTH_MARGIN + 5;
        rect.width  = messageTextView.bounds.width;
        rect.height = messageTextView.bounds.height;
        messageTextView.setBounds(rect);

        rect.y = max(messageTextView.bounds.maxY(), 
                        bitmapContainer.bounds.maxY()) + WIDTH_MARGIN + 4;

        /* Buttons */
        rect.x = buttonOffset;
        rect.width = buttonSize.width;
        rect.height = buttonSize.height;
        defaultButton.setBounds(rect);
        if( secondButton != null ) {
            rect.x += (rect.width + BUTTON_HORIZ_MARGIN);
            secondButton.setBounds(rect);
        }

        if( thirdButton != null ) {
            rect.x += (rect.width + BUTTON_HORIZ_MARGIN);
            thirdButton.setBounds(rect);
        }

        Size sz = new Size();
        sz.width = minWidth;
        sz.height = rect.y + buttonSize.height + BOTTOM_MARGIN;
        contentView.sizeTo(sz.width,sz.height);
    }

    private int run() {
        Size minSize;

        calculateLayout();

        window = new ExternalWindow( Window.TITLE_TYPE);
        window.setTitle(title);
        
        window.setResizable(false);
        minSize = window.windowSizeForContentSize( contentView.bounds.width,
                                            contentView.bounds.height);
        window.sizeTo(minSize.width,minSize.height);
        window.addSubview( contentView );
        window.rootView().setDefaultSelectedView(defaultButton);
        window.center();
        window.showModally();
        return result;
    }

    private int max(int i,int j) {
        if( i > j )
            return i;
        else
            return j;
    }

    private Size requiredSizeToPreserveLinesInTextView(TextView aTextView) {
        FontMetrics fm = new FontMetrics(aTextView.font());
        String sb = "";
        int i,c;
        int maxLineWidth = 0;
        byte ch;
        int lineCount=1;
        String text = aTextView.string();
        byte [] charArray;
        
        charArray = text.getBytes();
        for(i=0,c=text.length() ; i < c ; i++ ) {
            if((ch= charArray[i]) == '\n' ) {
                maxLineWidth = max( maxLineWidth , fm.stringWidth(sb));
                sb = "";
                lineCount++;
            } else
                sb += (char)(ch);
        }

        if( sb.length() != 0 )
            maxLineWidth = max(maxLineWidth, fm.stringWidth(sb.toString()));

        Size result = new Size();
        result.width = maxLineWidth;
        result.height = fm.stringHeight() * lineCount;
        return result;
    }

    void setResult(int aResult) {
        result = aResult;
    }

    void hide() {
        window.hide();
    }
}

class AlertContentView extends View implements Target {
    WinAlert alert;

    public AlertContentView(WinAlert anAlert,int x, int y, int w, int h) {
        super(x,y,w,h);
        alert = anAlert;
    }

    public void drawView(Graphics g) {
        g.setColor(WinColor.rootView);
        g.fillRect(g.clipRect());
    }

    public void performCommand(String command, Object data) {
        if (WinAlert.DEFAULT_ACTION.equals(command))
            alert.setResult(WinAlert.DEFAULT_OPTION);
        else if (WinAlert.SECOND_ACTION.equals(command))
            alert.setResult(WinAlert.SECOND_OPTION);
        else if (WinAlert.THIRD_ACTION.equals(command))
            alert.setResult(WinAlert.THIRD_OPTION);
        else
            throw new NoSuchMethodError("unknown command: " + command);

        alert.hide();
    }
}
