// WinCheckImage.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

// XXX The Arrow drawing code has been tweaked for 16x16.  It should be fixed
// when ScrollBars can be different sizes.

public class WinArrowImage extends Image {
    public static final int RIGHT_ARROW = 0;
    public static final int LEFT_ARROW = 1;
    public static final int DOWN_ARROW = 2;
    public static final int UP_ARROW = 3;

    private static WinArrowImage rightArrow;
    private static WinArrowImage leftArrow;
    private static WinArrowImage downArrow;
    private static WinArrowImage upArrow;
    private static WinArrowImage disabledDownArrow;

    int direction;

    Color arrowColor = WinColor.controlText;

    public static WinArrowImage rightArrow() {
        if (rightArrow == null) {
            rightArrow = new WinArrowImage(RIGHT_ARROW);
        }

        return rightArrow;
    }

    public static WinArrowImage leftArrow() {
        if (leftArrow == null) {
            leftArrow = new WinArrowImage(LEFT_ARROW);
        }

        return leftArrow;
    }

    public static WinArrowImage disabledDownArrow() {
        if (disabledDownArrow == null) {
            disabledDownArrow = new WinArrowImage(DOWN_ARROW);
            disabledDownArrow.setArrowColor(WinColor.controlShadow);
        }

        return disabledDownArrow;
    }

    public static WinArrowImage downArrow() {
        if (downArrow == null) {
            downArrow = new WinArrowImage(DOWN_ARROW);
        }

        return downArrow;
    }

    public static WinArrowImage upArrow() {
        if (upArrow == null) {
            upArrow = new WinArrowImage(UP_ARROW);
        }

        return upArrow;
    }

    private WinArrowImage(int direction) {
        super();
        this.direction = direction;
    }

    public int width() {
        return 12;
    }

    public int height() {
        return 12;
    }

    public void setArrowColor(Color color) {
        arrowColor = color;
    }

    public void drawAt(Graphics g, int x, int y) {
        int len, width, height;

        width = width();
        height = height();

        g.setColor(arrowColor);

        switch (direction) {
            case RIGHT_ARROW:
                x = x + width / 3;
                y = y + 3;
                len = height - 4;
                if ((len % 2) == 0)
                    len -= 1;

                while (len > 0) {
                    g.fillRect(x, y, 1, len);
                    x++;
                    y++;
                    len -= 2;
                }

                break;
            case LEFT_ARROW:
                x = x + 2 * (width / 3) - 2;
                y = y + 3;
                len = height - 4;
                if ((len % 2) == 0)
                    len -= 1;

                while (len > 0) {
                    g.fillRect(x, y, 1, len);
                    x--;
                    y++;
                    len -= 2;
                }

                break;
            case DOWN_ARROW:
                x = x + 2;
                y = y + height / 3 + 1;
                len = height - 4;
                if ((len % 2) == 0)
                    len -= 1;

                while (len > 0) {
                    g.fillRect(x, y, len, 1);
                    x++;
                    y++;
                    len -= 2;
                }

                break;
            case UP_ARROW:
                x = x + 2;
                y = y + 2 * height / 3;
                len = height - 4;
                if ((len % 2) == 0)
                    len -= 1;

                while (len > 0) {
                    g.fillRect(x, y, len, 1);
                    x++;
                    y--;
                    len -= 2;
                }

                break;
            default:
                throw new RuntimeException("Unknown arrow direction: " +
                    direction);
        }
    }

    public boolean isTransparent() {
        return true;
    }
}
