// WinCheckImage.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

public class WinCheckImage extends Image {
    private static WinCheckImage checkedImage;
    private static WinCheckImage uncheckedImage;
    private static WinCheckImage disabledCheckedImage;
    private static WinCheckImage disabledUncheckedImage;

    boolean checked;
    boolean enabled = true;

    public static WinCheckImage checkedImage() {
        if(checkedImage == null)
            checkedImage = new WinCheckImage(true);
        return checkedImage;    
    }

    public static WinCheckImage uncheckedImage() {
        if(uncheckedImage == null)
            uncheckedImage = new WinCheckImage(false);
        return uncheckedImage;  
    }

    public static WinCheckImage disabledCheckedImage() {
        if(disabledCheckedImage == null) {
            disabledCheckedImage = new WinCheckImage(true);
            disabledCheckedImage.setEnabled(false);
        }
        return disabledCheckedImage;
    }

    public static WinCheckImage disabledUncheckedImage() {
        if(disabledUncheckedImage == null) {
            disabledUncheckedImage = new WinCheckImage(false);
            disabledUncheckedImage.setEnabled(false);
        }
        return disabledUncheckedImage;
    }

    private WinCheckImage(boolean checked) {
        super();
        this.checked = checked;
    }

    public int width() {
        return 15;
    }

    public int height() {
        return 14;
    }

    public void setEnabled(boolean value) { enabled = value; }

    public void drawAt(Graphics g, int x, int y) {
        int width, height;

        // XXX Hack to get the image to appear in the right place on the check
        // button.

        width = width() - 2;
        height = height() - 1;
        y += 1;

        WinBorder.loweredBezel().drawInRect(g, x, y, width, height);
        if(enabled)
            g.setColor(WinColor.text);
        else
            g.setColor(WinColor.control);
        g.fillRect(x + 2, y + 2, width - 4, height - 4);
        if (checked) {
            x += 3;
            y += 5;
            if(enabled)
                g.setColor(WinColor.textText);
            else
                g.setColor(WinColor.textInactiveText);
            
            g.fillRect(x++, y++, 1, 3);
            g.fillRect(x++, y++, 1, 3);
            g.fillRect(x++, y--, 1, 3);
            g.fillRect(x++, y--, 1, 3);
            g.fillRect(x++, y--, 1, 3);
            g.fillRect(x++, y--, 1, 3);
            g.fillRect(x++, y--, 1, 3);
        }
    }

    public boolean isTransparent() {
        return true;
    }
}
