// WinComboBox.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

public class WinComboBox extends View implements Target {
    public static String ITEM_SELECTED_COMMAND = "itemSelected";
    public static String WILL_SHOW_WINDOW = "willShowWindow";

    WinTextField textField;
    WinButton dropButton;
    Border comboBorder;

    InternalWindow dropWindow;
    ContainerView container;
    ListView dropList;

    Target comboTarget;
    String comboCommand;

    Vector titles;

    boolean enabled = true;
    boolean downInListView = false;
    boolean reloadOnShow = false;

    public WinComboBox() {
        // Using some 'normal' values for w/h as the createComponents() code
        // dislikes having to deal with 0, 0 values.
        this(0, 0, 180, 22);
    }

    public WinComboBox(int x, int y, int w, int h) {
        super(x, y, w, h);

        createComponents();
    }

    public void createComponents() {
        Vector defaultTitles;

        dropButton = new WinButton(bounds.width - 2 - 16, 2, 16,
            bounds.height - 4);
        dropButton.setRaisedBorder(WinBorder.raisedScrollButton());
        dropButton.setLoweredBorder(WinBorder.loweredScrollButton());
        dropButton.setImage(WinArrowImage.downArrow());
        dropButton.setAltImage(WinArrowImage.downArrow());
        dropButton.setHorizResizeInstruction(LEFT_MARGIN_CAN_CHANGE);
        dropButton.setVertResizeInstruction(HEIGHT_CAN_CHANGE);
        addSubview(dropButton);

        textField = new WinTextField(4, 2, dropButton.bounds.x - 4,
            bounds.height - 4);
        textField.setBorder(null);
        textField.setEditable(false);
        textField.setHorizResizeInstruction(WIDTH_CAN_CHANGE);
        textField.setVertResizeInstruction(HEIGHT_CAN_CHANGE);
        addSubview(textField);

        comboBorder = WinBorder.loweredBezel();

        // Components for the drop down window

        dropWindow = new InternalWindow(0, 0, bounds.width, bounds.height);
        dropWindow.setType(InternalWindow.BLANK_TYPE);
        dropWindow.setLayer(InternalWindow.POPUP_LAYER);

        container = new ContainerView(0, 0, bounds.width, bounds.height);
        container.setTransparent(true);
        container.setBorder(LineBorder.blackLine()); // XXX
        container.setVertResizeInstruction(HEIGHT_CAN_CHANGE);
        container.setHorizResizeInstruction(WIDTH_CAN_CHANGE);
        dropWindow.addSubview(container);

        dropList = new ListView(1, 1, bounds.width - 1, bounds.height - 1);
        dropList.setPrototypeItem(new WinListItem());
        dropList.setTarget(this);
        dropList.setCommand(ITEM_SELECTED_COMMAND);
        dropList.setAllowsEmptySelection(true);
        dropList.setAllowsMultipleSelection(false);
        container.addSubview(dropList);

        defaultTitles = new Vector(1);
        defaultTitles.addElement("");
        setTitles(defaultTitles);
        selectTitle("");
    }

    public void setReloadOnShow(boolean flag) {
        reloadOnShow = flag;
    }

    public boolean reloadOnShow() {
        return reloadOnShow;
    }

    public boolean enabled() {
        return enabled;
    }

    public void setEnabled(boolean enabled) {
        if (this.enabled == enabled)
            return;

        this.enabled = enabled;

        if (enabled) {
            textField.setTextColor(WinColor.textText);
            dropButton.setImage(WinArrowImage.downArrow());
        } else {
            textField.setTextColor(WinColor.textInactiveText);
            dropButton.setImage(WinArrowImage.disabledDownArrow());
        }
        setDirty(true);
    }

    public void setTarget(Target target) {
        comboTarget = target;
    }

    public void setCommand(String command) {
        comboCommand = command;
    }

    static boolean isASCII(char b) {
        return ((b >= 32) && (b <= 126));
    }

    static String asciiTitle(String title) {
        StringBuffer newTitle = null;
        char chars[] = new char[title.length()];
        int length = chars.length, i;

        title.getChars(0, length, chars, 0);
        for (i = 0; i < length; i++) {
            char c = chars[i];

            if (!isASCII(c)) {
                if (newTitle == null) {
                    newTitle = new StringBuffer(title);
                }
                newTitle.setCharAt(i, '?');
            }
        }
        return newTitle != null ? new String(newTitle) : title;
    }

    /** This takes a vector of strings and sets them as the titles of the
      * items in the ComboBox.
      */
    public void setTitles(Vector titles) {
        int i, count;

        this.titles = (Vector)titles.clone();

        dropList.removeAllItems();

        count = titles.count();
        for (i = 0; i < count; i++) {
            String title = (String)titles.elementAt(i);
            String asciiTitle = asciiTitle(title);

            dropList.addItem().setTitle(asciiTitle);
        }

        dropList.selectItem(null);
    }

    public void selectTitle(String title) {
        int i, count;
        String t;

        count = titles.count();

        for (i = 0; i < count; i++) {
            t = (String)titles.elementAt(i);
            if (title.equals(t)) {
                dropList.selectItemAt(i);
                textField.setStringValue(asciiTitle(title));
                return;
            }
        }
    }

    public String selectedTitle() {
        return (String)titles.elementAt(dropList.selectedIndex());
    }

    public void drawView(Graphics g) {
        comboBorder.drawInRect(g, 0, 0, bounds.width, bounds.height);

        // Fill in the little turdlet of space
        g.setColor(WinColor.text);
        g.fillRect(2, 2, 2, bounds.height - 4);
    }

    public View viewForMouse(int x, int y) {
        if (containsPoint(x, y))
            return this;

        return null;
    }

    public boolean mouseDown(MouseEvent event) {
        if (!enabled)
            return false;

        if (reloadOnShow && comboTarget != null) {
            comboTarget.performCommand(WILL_SHOW_WINDOW, this);
        }
        layoutDropWindow();
        showDropWindow(event);

        runModalLoop();

        return true;
    }

    protected void layoutDropWindow() {
        Rect    windowRect;
        int     selectedIndex;

        dropList.setBounds(1, 1, bounds.width - 2,
            dropList.rowHeight() * dropList.count());

        windowRect = new Rect(0, height(), width(), dropList.height() + 2);
        convertRectToView(null, windowRect, windowRect);
        dropWindow.setBounds(windowRect);
    }

    protected void showDropWindow(MouseEvent event) {
        dropButton.setState(true);
        dropWindow.setRootView(rootView());

        dropWindow.show();
    }

    private void runModalLoop() {
        EventLoop loop;
        Event event;
        MouseEvent mouseEvent;
        MouseEvent dropListEvent;
        boolean processEvent;
        boolean inWindow;

        loop = Application.application().eventLoop();
        downInListView = false;

        while (dropWindow.isVisible() && loop.isRunning()) {
            event = loop.getNextEvent();
            processEvent = true;

            if (event instanceof MouseEvent) {
                mouseEvent = (MouseEvent)event;
                inWindow = dropWindow.bounds.contains(
                                        mouseEvent.x, mouseEvent.y);
                switch(mouseEvent.type()) {
                case MouseEvent.MOUSE_DOWN:
                    if(!inWindow) {
                        processEvent = false;
                        dropWindow.hide();
                    }
                    break;
                case MouseEvent.MOUSE_MOVED:
                case MouseEvent.MOUSE_DRAGGED:
                    if(inWindow) {
                        dropListEvent = rootView().convertEventToView(
                                            dropList, mouseEvent);
                        processEvent = false;
                        if(downInListView)
                            dropList.mouseDragged(dropListEvent);
                        else {
                            downInListView = true;
                            dropList.mouseDown(dropListEvent);
                        }
                    }
                    break;
                case MouseEvent.MOUSE_UP:
                  if(inWindow && downInListView) {
                        processEvent = false;
                        downInListView = false;
                        performCommand(ITEM_SELECTED_COMMAND, null);
                  }
                  break;
                }
            }

            if (processEvent) {
                try {
                    loop.processEvent(event);
                } catch (Exception e) {
                    System.err.println("Uncaught ComboBox Exception.");
                    e.printStackTrace(System.err);
                    System.err.println("Restarting ComboBox EventLoop.");
                }
            }
        }
// Because we are doing a mouseDown() in the ListView during a move,
// the ifc mouse state machinery gets confused. The result, I currently
// believe, is that the ListView continues getting mouseEvents after the
// internal window hides(). By reseting the mouseView to be the rootView
// when we hide, the proper behavior seems to work better.
        rootView().setMouseView(rootView());
        downInListView = false;
        dropButton.setState(false);
    }

    public void itemSelected() {
        String title = selectedTitle(),
               asciiTitle = dropList.selectedItem().title();

        if(title != null) {
            textField.setStringValue(asciiTitle);

            if (comboTarget != null)
                comboTarget.performCommand(comboCommand, title);
        }

        dropWindow.hide();
    }

    // Target

    public void performCommand(String command, Object data) {
        if (ITEM_SELECTED_COMMAND.equals(command))
            itemSelected();
        else
            throw new RuntimeException("Unknown command: " + command);
    }

    /** Prevent the IFC keyboard UI from poking around inside the combo box.
      */
    public boolean hidesSubviewsFromKeyboard() {
        return true;
    }
}
