// WinListItem.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

public class WinListItem extends ListItem {
    static Bitmap focusedRectBitmap = null;

    boolean isFocused = false;

    public WinListItem() {
        super();
    }

    public int minWidth() {
        return super.minWidth();
    }

    public int minHeight() {
        return super.minHeight();
    }

    public boolean isTransparent() {
        return false;
    }

    public void setFocused(boolean value) { isFocused = value; } 
    public boolean isFocused() { return isFocused; } 

    public void drawInRect(Graphics g, Rect bounds) {
        Color oldColor = g.color();

        if (isSelected()) {
            if(isFocused) {
                drawDashedRect(g, bounds);
            } else {
                g.setColor(WinColor.textHighlight);
                g.fillRect(bounds);
            }
           g.setColor(WinColor.textHighlightText);
        } else {
            g.setColor(WinColor.text);
            g.fillRect(bounds);
            g.setColor(WinColor.textText);
        }

        g.setFont(WinFont.defaultFont());

        // XXX I forget whether you are allowed to mutate bounds and the
        // current color.  Just to be safe...

        bounds.x++;
        g.drawStringInRect(title(), bounds, Graphics.LEFT_JUSTIFIED);
        bounds.x--;

        g.setColor(oldColor);
    }

    public void drawDashedRect(Graphics g, Rect r) {
        g.drawBitmapAt(focusedRectBitmap(r.width, r.height), r.x, r.y);
    }

    static Bitmap focusedRectBitmap(int w, int h) {
        Graphics g;

        if(focusedRectBitmap != null
            && focusedRectBitmap.width() == w
            && focusedRectBitmap.height() == h) {
            return focusedRectBitmap;
        }

        focusedRectBitmap = new Bitmap(w, h);
        g = new Graphics(focusedRectBitmap);

        g.setColor(WinColor.textHighlight);
        g.fillRect(0, 0, w, h);
        _drawDashedRect(g, new Rect(0, 0, w, h));

        g.dispose();
        return focusedRectBitmap;
    }

    static void _drawDashedRect(Graphics g, Rect r) {
        int _x,_y, bx, by, bw, bh;
        
        g.setColor(WinColor.controlHighlight);

        bx = r.x;
        by = r.y;
        bw = r.width;
        bh = r.height;

        // we always draw on odd boundries, the lines look better,
        if((bw % 2) == 0)
           bw--;

        if((bh % 2) == 0)
           bh--;

        // draw upper and lower horizontal dashes
        for (_x = bx; _x < (bx + bw); _x += 2) {
            g.drawLine(_x, by, _x, by);
            g.drawLine(_x, by + bh - 1, _x, by + bh - 1);
        }

        // draw left and right vertical dashes
        for (_y = by; _y < (by + bh ); _y += 2) {
            g.drawLine(bx, _y, bx, _y);
            g.drawLine(bx + bw - 1, _y, bx + bw - 1, _y);
        }

    }

}
