// WinListView.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

public class WinListView extends ListView {
    ListItem _anchorItem;
    boolean  hasFocus;
    int pageSize;

    static final String SHIFT_NEXT_ITEM = "shiftNextItem";
    static final String SHIFT_PREV_ITEM = "shiftPrevItem";

    static final String HOME = "home";
    static final String END = "end";
    static final String PAGE_UP = "page_up";
    static final String PAGE_DOWN = "page_down";

    static final String SHIFT_HOME = "shiftHome";
    static final String SHIFT_END = "shiftEnd";
    static final String SHIFT_PAGE_UP = "shiftPage_up";
    static final String SHIFT_PAGE_DOWN = "shiftPage_down";

    public WinListView(int x, int y, int width, int height) {
        this(new Rect(x, y, width, height));        
    }

    public WinListView(Rect bounds) {
        super(bounds);
        
        removeAllCommandsForKeys();
        setCommandForKey(SELECT_NEXT_ITEM,KeyEvent.DOWN_ARROW_KEY,View.WHEN_SELECTED);
        setCommandForKey(SELECT_PREVIOUS_ITEM,KeyEvent.UP_ARROW_KEY,View.WHEN_SELECTED);
        setCommandForKey(SHIFT_NEXT_ITEM, null, KeyEvent.DOWN_ARROW_KEY, KeyEvent.SHIFT_MASK,View.WHEN_SELECTED);
        setCommandForKey(SHIFT_PREV_ITEM, null, KeyEvent.UP_ARROW_KEY,KeyEvent.SHIFT_MASK, View.WHEN_SELECTED);

        setCommandForKey(HOME,KeyEvent.HOME_KEY,View.WHEN_SELECTED);
        setCommandForKey(END,KeyEvent.END_KEY,View.WHEN_SELECTED);
        setCommandForKey(SHIFT_HOME, null, KeyEvent.HOME_KEY, KeyEvent.SHIFT_MASK,View.WHEN_SELECTED);
        setCommandForKey(SHIFT_END, null, KeyEvent.END_KEY,KeyEvent.SHIFT_MASK, View.WHEN_SELECTED);

        setCommandForKey(PAGE_UP,KeyEvent.PAGE_UP_KEY,View.WHEN_SELECTED);
        setCommandForKey(PAGE_DOWN,KeyEvent.PAGE_DOWN_KEY,View.WHEN_SELECTED);
        setCommandForKey(SHIFT_PAGE_UP, null, KeyEvent.PAGE_UP_KEY, KeyEvent.SHIFT_MASK,View.WHEN_SELECTED);
        setCommandForKey(SHIFT_PAGE_DOWN, null, KeyEvent.PAGE_DOWN_KEY,KeyEvent.SHIFT_MASK, View.WHEN_SELECTED);

    }

    public void setPageSize(int pageSize) {
        this.pageSize = pageSize;
    }

    public int pageSize() {
        return pageSize;
    }

    public boolean mouseDown(MouseEvent event) {
        ListItem    clickedItem;
        boolean result;

        if (!isEnabled()) {
            return false;
        }

        clickedItem = itemForPoint(event.x, event.y);

        if(!(event.isControlKeyDown() || event.isShiftKeyDown())) {
            _anchorItem = clickedItem;
            result = super.mouseDown(event);
            updateFocus();
            return result;
        }

        if (clickedItem == null)
            return true;

        if(event.isControlKeyDown()) {
            if(clickedItem.isSelected())    {
                deselectItem(clickedItem);
            } else {        
                selectItem(clickedItem);
            }
        } else if(event.isShiftKeyDown()) {
            int curIndex, newIndex;
            int i, count;

            curIndex = indexOfItem(_anchorItem);
            newIndex = indexOfItem(clickedItem);

            disableDrawing();

            selectOnly(_anchorItem);    // Clear any other selection
            
            if(curIndex < newIndex) {
                for(i = curIndex; i <= newIndex;i++)
                    selectItemAt(i);
            } else if(newIndex < curIndex) {
                for(i = newIndex; i <= curIndex;i++)
                    selectItemAt(i);
            }       

            reenableDrawing();
            setDirty(true);
        }

        if (event.clickCount() == 2) {
            sendDoubleCommand();
            updateFocus();
// I think we only want to track the anchor on a single click (super.mouseDown() call only
//          _anchorItem = clickedItem;      
            return false;
        }

// I think we only want to track the anchor on a single click (super.mouseDown() call only
//      _anchorItem = clickedItem;
        updateFocus();
        return true;
    }

    public void performCommand(String aCommand,Object anObject) {
        if(SELECT_NEXT_ITEM.equals(aCommand)) 
            _selectNextItem(true, false);
         else if(SELECT_PREVIOUS_ITEM.equals(aCommand))
            _selectNextItem(false, false);
         else if(SHIFT_NEXT_ITEM.equals(aCommand))
            _selectNextItem(true, true);
         else if(SHIFT_PREV_ITEM.equals(aCommand))
            _selectNextItem(false, true);
         else if(HOME.equals(aCommand))
            selectHome(false);
         else if(END.equals(aCommand))
            selectEnd(false);
         else if(PAGE_UP.equals(aCommand))
            page(pageSize, false);
         else if(PAGE_DOWN.equals(aCommand))
            page(-pageSize, false);
         else if(SHIFT_HOME.equals(aCommand))
            selectHome(true);
         else if(SHIFT_END.equals(aCommand))
            selectEnd(true);
         else if(SHIFT_PAGE_UP.equals(aCommand))
            page(pageSize, true);
         else if(SHIFT_PAGE_DOWN.equals(aCommand))
            page(-pageSize, true);
        updateFocus();
    }

    public void page(int amount, boolean shift) {
        int i, count;
        boolean forward;
        int numRows;

        if(count() < 1)
            return;

        if(superview() instanceof ScrollView)
            ((ScrollView)superview()).scrollBy(0, amount);

        if(!shift)
            return;

        numRows = pageSize / rowHeight();

        disableDrawing();       
        forward = !(amount > 0);
        for(i=0; i < numRows; i++)
            _selectNextItem(forward, true);
        reenableDrawing();

        draw();
    }

    public void selectHome(boolean shift) {
        int indexOfAnchor;
        int i, count;

        if(count() < 1)
            return;

        if(!shift) {
            _anchorItem = itemAt(0);
            selectOnly(_anchorItem);
            scrollItemToVisible(_anchorItem);   
            return;
        }

        indexOfAnchor = indexOfItem(_anchorItem);       
        for(i = 0; i <= indexOfAnchor; i++) {
            if(i == 0)
                selectOnly(itemAt(0));
            else
                selectItemAt(i);
        }
        
        scrollItemAtToVisible(0);       
    }

    public void selectEnd(boolean shift) {
        int indexOfAnchor, max;
        int i, count;

        if(count() < 1)
            return;

        if(!shift) {
            _anchorItem = itemAt(count()-1);
            selectOnly(_anchorItem);
            scrollItemToVisible(_anchorItem);
            return;
        }
        
        indexOfAnchor = indexOfItem(_anchorItem);       
        max = count();
        for(i = indexOfAnchor; i < max; i++) {
            if(i == indexOfAnchor)
                selectOnly(_anchorItem);
            else
                selectItemAt(i);
        }
        
        scrollItemAtToVisible(max-1);           
    }

    void _selectNextItem(boolean forward, boolean shift) {
        Vector si = selectedItems();
        int sic = si.count();
        int indexOfAnchor;
        int farthestAwayIndex = -1;
        int i, count;
        int nextIndex;
        ListItem deltaItem = null;

        count = count();

        if(count <= 0)
            return;

        // Special case the empty set
        if(si.count() <= 0) {
            selectItemAt(0);
            sendCommand();
            scrollRectToVisible(rectForItem(itemAt(0)));
            _anchorItem = itemAt(0);
            return;
        }

        indexOfAnchor = indexOfItem(_anchorItem);
        farthestAwayIndex = indexOfAnchor;
        count = sic;
        for(i=0; i < count; i++) {
            nextIndex = indexOfItem(((ListItem)si.elementAt(i)));
            if(Math.abs(indexOfAnchor - nextIndex) > Math.abs(indexOfAnchor - farthestAwayIndex)) {
                farthestAwayIndex = nextIndex;
            }
        }

        // Special case out the non-shift case
        if(!shift) {
            int newIndex = 0;
            count = count();
            indexOfAnchor = farthestAwayIndex;

            if(forward) {
                if(indexOfAnchor <= count - 2)
                    newIndex = indexOfAnchor + 1;
                else
                    newIndex = indexOfAnchor;
            } else {
                if(indexOfAnchor > 0)
                    newIndex = indexOfAnchor - 1;
                else
                    newIndex = 0;
            }
            _anchorItem = itemAt(newIndex);
            selectOnly(_anchorItem);
            sendCommand();
            scrollRectToVisible(rectForItemAt(newIndex));
            return;
        }

        // Do the hard work
        count = count() - 1;
        if(farthestAwayIndex < indexOfAnchor) {
            if(forward) {
                 deltaItem = itemAt(farthestAwayIndex);
                 deselectItem(deltaItem);
            } else {
                if(farthestAwayIndex > 0) {
                    deltaItem = itemAt(farthestAwayIndex-1);
                    selectItem(deltaItem);
                }
            }
        } else if(farthestAwayIndex > indexOfAnchor) {
            if(forward) {
                if(farthestAwayIndex+1 <= count) {
                    deltaItem = itemAt(farthestAwayIndex+1);
                    selectItem(deltaItem);
                }
            } else {
                if(farthestAwayIndex > 0) {
                    deltaItem = itemAt(farthestAwayIndex);
                    deselectItem(deltaItem);
                }
            }   
        } else if(farthestAwayIndex == indexOfAnchor) {
            if(forward) {
                if(farthestAwayIndex+1 <= count) {
                    deltaItem = itemAt(farthestAwayIndex+1);
                    selectItem(deltaItem);
                }
            } else {
                if(farthestAwayIndex > 0) {
                    deltaItem = itemAt(farthestAwayIndex-1);
                    selectItem(deltaItem);
                }
            }   
        }

        if(deltaItem != null) {
            scrollRectToVisible(rectForItem(deltaItem));
            sendCommand();      
        }

    }

    /** Overridden to remove the request for the keyboard Arrow
      */
    public void willBecomeSelected() {
        hasFocus = true;
        updateFocus();
    }

    /** Overridden to remove the request for the keyboard Arrow
      */
    public void willBecomeUnselected() {
        hasFocus = false;
        updateFocus();
    }

    public void selectOnly(ListItem item) {
        super.selectOnly(item);
        _anchorItem = item;
        updateFocus();
    }

    public void updateFocus() {
        int i, count;
        WinListItem wi;
        
        count = count();
        for(i = 0; i < count; i++) {
            wi = (WinListItem)itemAt(i);
            if((wi.isFocused() && !wi.equals(selectedItem())) 
                || (wi.equals(selectedItem()) && !hasFocus)) {
                wi.setFocused(false);
                drawItemAt(i);
            }
        }
        if(hasFocus && selectedItem() != null && !((WinListItem)selectedItem()).isFocused()) {
            ((WinListItem)selectedItem()).setFocused(true);
            drawItemAt(selectedIndex());
        }
    }
}

