// WinScrollBar.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

// XXX The scroll bar needs to get the default windows size.  Hunt down all
// the 16s.

public class WinScrollBar extends ScrollBar {
    private static Bitmap scrollTrayImage;

    public static Image scrollTrayImage() {
        int x;
        Graphics g;

        if (scrollTrayImage != null)
            return scrollTrayImage;

        scrollTrayImage = new Bitmap(16, 16);
        g = new Graphics(scrollTrayImage);
        g.setColor(WinColor.scrollbar);
        g.fillRect(0, 0, 16, 16);

        g.setColor(WinColor.controlLtHighlight);

        for (x = -16; x < 16; x += 2) {
            g.drawLine(x, 0, x + 16, 16);
        }

        g.dispose();

        return scrollTrayImage;
    }

    public WinScrollBar(int x, int y, int w, int h, int axis) {
        super(x, y, w, h, axis);

        Button button;

        button = increaseButton();
        button.setRaisedColor(WinColor.control);
        button.setLoweredColor(WinColor.control);
        button.setTitleColor(WinColor.controlText);
        button.setFont(WinFont.defaultFont());
        button.setRaisedBorder(WinBorder.raisedScrollButton());
        button.setLoweredBorder(WinBorder.loweredScrollButton());
        if (axis == Scrollable.HORIZONTAL) {
            button.setImage(WinArrowImage.rightArrow());
            button.setAltImage(WinArrowImage.rightArrow());
        } else {
            button.setImage(WinArrowImage.downArrow());
            button.setAltImage(WinArrowImage.downArrow());
        }

        button = decreaseButton();
        button.setRaisedColor(WinColor.control);
        button.setLoweredColor(WinColor.control);
        button.setTitleColor(WinColor.controlText);
        button.setFont(WinFont.defaultFont());
        button.setRaisedBorder(WinBorder.raisedScrollButton());
        button.setLoweredBorder(WinBorder.loweredScrollButton());
        if (axis == Scrollable.HORIZONTAL) {
            button.setImage(WinArrowImage.leftArrow());
            button.setAltImage(WinArrowImage.leftArrow());
        } else {
            button.setImage(WinArrowImage.upArrow());
            button.setAltImage(WinArrowImage.upArrow());
        }

        // Always have buttons, even when disabled
        addParts();
    }

    public void removeParts() {
        // Always have buttons, even when disabled
    }

    public Size minSize() {
        // XXX
        return new Size(16, 16);
    }

    public void addParts() {
        Button decreaseButton, increaseButton;

        decreaseButton = decreaseButton();
        increaseButton = increaseButton();

        if (decreaseButton != null)
            decreaseButton.moveTo(0, 0);

        if (axis() == Scrollable.HORIZONTAL) {
            if (increaseButton != null)
                increaseButton.moveTo(width() - increaseButton.width(), 0);
        } else {
            if (increaseButton != null)
                increaseButton.moveTo(0, height() - increaseButton.height());
        }

        addSubview(increaseButton);
        addSubview(decreaseButton);
    }

    public Rect scrollTrayRect() {
        int x, maxX, y, maxY;
        Rect r;
        Button decreaseButton, increaseButton;

        decreaseButton = decreaseButton();
        increaseButton = increaseButton();

        if (decreaseButton == null || increaseButton == null)
            return new Rect(bounds);

        if (axis() == Scrollable.HORIZONTAL) {
            x = decreaseButton.width();
            maxX = width() - increaseButton.width();

            r = new Rect(x, 0, maxX - x, height());
        } else {
            y = decreaseButton.height();
            maxY = height() - increaseButton.height();

            r = new Rect(0, y, width(), maxY - y);
        }

        return r;
    }

    public void drawView(Graphics g) {
        Rect knobRect, trayRect, clipRect;

        // Draw the knob first.

        knobRect = knobRect();
        WinBorder.raisedScrollButton().drawInRect(g, knobRect);

        // Draw the areas outside the knob.

        trayRect = scrollTrayRect();
        clipRect = new Rect();

        if (axis() == Scrollable.HORIZONTAL) {
            clipRect.x = trayRect.x;
            clipRect.y = trayRect.y;
            clipRect.width = knobRect.x - clipRect.x;
            clipRect.height = trayRect.height;

            g.pushState();
            g.setClipRect(clipRect);
            scrollTrayImage().drawTiled(g, trayRect);
            g.popState();

            clipRect.x = knobRect.maxX();
            clipRect.width = trayRect.maxX() - clipRect.x;

            g.pushState();
            g.setClipRect(clipRect);
            scrollTrayImage().drawTiled(g, trayRect);
            g.popState();
        } else {
            clipRect.x = trayRect.x;
            clipRect.y = trayRect.y;
            clipRect.width = trayRect.width;
            clipRect.height = knobRect.y - clipRect.y;

            g.pushState();
            g.setClipRect(clipRect);
            scrollTrayImage().drawTiled(g, trayRect);
            g.popState();

            clipRect.y = knobRect.maxY();
            clipRect.height = trayRect.maxY() - clipRect.y;

            g.pushState();
            g.setClipRect(clipRect);
            scrollTrayImage().drawTiled(g, trayRect);
            g.popState();
        }
    }

    /** Overridden to remove the request for the keyboard Arrow
      */
    public void willBecomeSelected() {
    }

    /** Overridden to remove the request for the keyboard Arrow
      */
    public void willBecomeUnselected() {
    }

}
